/*
  Copyright 2002-2021 Google LLC
  Copyright 2002-2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package org.opengroup.osdu.register.logging;

import com.google.common.base.Strings;
import java.util.List;
import org.opengroup.osdu.core.common.logging.audit.AuditAction;
import org.opengroup.osdu.core.common.logging.audit.AuditPayload;
import org.opengroup.osdu.core.common.logging.audit.AuditStatus;

public class AuditEvents {

  private static final String CREATE_DMS_ACTION_ID = "RG10001";
  private static final String CREATE_DMS_MESSAGE = "Registered new DDMS";

  private static final String DELETE_DMS_ACTION_ID = "RG10002";
  private static final String DELETE_DMS_MESSAGE = "Deleted DDMS";

  private static final String CREATE_SUBSCRIPTION_ACTION_ID = "RG10003";
  private static final String CREATE_SUBSCRIPTION_MESSAGE = "Create new subscription";

  private static final String DELETE_SUBSCRIPTION_ACTION_ID = "RG10004";
  private static final String DELETE_SUBSCRIPTION_MESSAGE = "Deleted subscription";

  private static final String UPDATE_SUBSCRIPTION_SECRET_ACTION_ID = "RG10005";
  private static final String UPDATE_SUBSCRIPTION_SECRET_MESSAGE = "Updated subscription";

  private static final String READ_ACTION_ACTION_ID = "RG10006";
  private static final String READ_ACTION_MESSAGE = "Read action";

  private static final String READ_MULTIPLE_ACTIONS_ACTION_ID = "RG10007";
  private static final String READ_MULTIPLE_ACTIONS_MESSAGE = "Read multiple actions";

  private static final String CREATE_ACTION_ACTION_ID = "RG10008";
  private static final String CREATE_ACTION_MESSAGE = "Create new action";

  private static final String DELETE_ACTION_ACTION_ID = "RG10009";
  private static final String DELETE_ACTION_MESSAGE = "Deleted action";

  private static final String VALIDATE_ACTION_ACTION_ID = "RG10010";
  private static final String VALIDATE_ACTION_MESSAGE = "Validate action";

  private static final String READ_SUBSCRIPTION_ACTION_ID = "RG10011";
  private static final String READ_SUBSCRIPTION_MESSAGE = "Read subscription";


  private final String user;


  public AuditEvents(String user) {
    if (Strings.isNullOrEmpty(user)) {
      throw new IllegalArgumentException("User not provided for audit events.");
    }
    this.user = user;
  }

  public AuditPayload getCreateDmsEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.CREATE)
        .status(status)
        .user(this.user)
        .actionId(CREATE_DMS_ACTION_ID)
        .message(getStatusMessage(status, CREATE_DMS_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getDeleteDmsEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.DELETE)
        .status(status)
        .user(this.user)
        .actionId(DELETE_DMS_ACTION_ID)
        .message(getStatusMessage(status, DELETE_DMS_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getCreateSubscriptionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.CREATE)
        .status(status)
        .user(this.user)
        .actionId(CREATE_SUBSCRIPTION_ACTION_ID)
        .message(getStatusMessage(status, CREATE_SUBSCRIPTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getDeleteSubscriptionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.DELETE)
        .status(status)
        .user(this.user)
        .actionId(DELETE_SUBSCRIPTION_ACTION_ID)
        .message(getStatusMessage(status, DELETE_SUBSCRIPTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getUpdateSubscriptionSecretEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.UPDATE)
        .status(status)
        .user(this.user)
        .actionId(UPDATE_SUBSCRIPTION_SECRET_ACTION_ID)
        .message(getStatusMessage(status, UPDATE_SUBSCRIPTION_SECRET_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getReadActionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.READ)
        .status(status)
        .user(this.user)
        .actionId(READ_ACTION_ACTION_ID)
        .message(getStatusMessage(status, READ_ACTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getReadMultipleActionsEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.READ)
        .status(status)
        .user(this.user)
        .actionId(READ_MULTIPLE_ACTIONS_ACTION_ID)
        .message(getStatusMessage(status, READ_MULTIPLE_ACTIONS_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getCreateActionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.CREATE)
        .status(status)
        .user(this.user)
        .actionId(CREATE_ACTION_ACTION_ID)
        .message(getStatusMessage(status, CREATE_ACTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getDeleteActionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.DELETE)
        .status(status)
        .user(this.user)
        .actionId(DELETE_ACTION_ACTION_ID)
        .message(getStatusMessage(status, DELETE_ACTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getValidateActionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.READ)
        .status(status)
        .user(this.user)
        .actionId(VALIDATE_ACTION_ACTION_ID)
        .message(getStatusMessage(status, VALIDATE_ACTION_MESSAGE))
        .resources(resources)
        .build();
  }

  public AuditPayload getReadSubscriptionEvent(AuditStatus status, List<String> resources) {
    return AuditPayload.builder()
        .action(AuditAction.READ)
        .status(status)
        .user(this.user)
        .actionId(READ_SUBSCRIPTION_ACTION_ID)
        .message(getStatusMessage(status, READ_SUBSCRIPTION_MESSAGE))
        .resources(resources)
        .build();
  }

  private String getStatusMessage(AuditStatus status, String message) {
    return String.format("%s - %s", message, status.name().toLowerCase());
  }
}
