/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.services;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.opengroup.osdu.register.action.model.CreateActionDto;
import org.opengroup.osdu.register.action.model.Filter;
import org.opengroup.osdu.register.action.model.ParsedAction;
import org.opengroup.osdu.register.action.model.TestActionRequest;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class TestActionServiceTest {

    @Mock
    private ParseJsonIntoAction actionParser;
    @InjectMocks
    private TestActionService sut;

    @Test
    public void should_validateRegex_given_validInput() {
        Map<String, Object> data = new HashMap<>();
        data.put("uri", "https://myproj.com/abc123");
        Map<String, Object> record = new HashMap<>();
        record.put("kind", "common:petrel:regularheightfield:1.0.0");
        record.put("data", data);
        ObjectMapper mapper = new ObjectMapper();
        JsonNode jsonNode = mapper.convertValue(record, JsonNode.class);
        TestActionRequest testActionRequest = TestActionRequest.builder().action(getCreateActionDto()).testPayload(jsonNode).build();

        when(this.actionParser.parse("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}", jsonNode)).thenReturn(ParsedAction.builder().url("https://myapp.osdu.opengroup.org/action/abc123").build());

        ParsedAction response = this.sut.validateRegex(testActionRequest);
        assertEquals("https://myapp.osdu.opengroup.org/action/abc123", response.getUrl());
    }

    @Test
    public void should_throwError_given_missingKindInPayload() {
        Map<String, Object> data = new HashMap<>();
        data.put("uri", "https://myproj.com/abc123");
        Map<String, Object> record = new HashMap<>();
        record.put("data", data);
        ObjectMapper mapper = new ObjectMapper();
        JsonNode jsonNode = mapper.convertValue(record, JsonNode.class);
        TestActionRequest testActionRequest = TestActionRequest.builder().action(getCreateActionDto()).testPayload(jsonNode).build();

        ParsedAction response = this.sut.validateRegex(testActionRequest);
        assertEquals("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}", response.getUrl());
        assertEquals("Filter did not match the testPayload.", response.getErrors());
    }

    @Test
    public void should_throwError_given_nonStringKind() {
        Map<String, Object> kind = new HashMap<>();
        kind.put("dummy", 122);
        Map<String, Object> data = new HashMap<>();
        data.put("uri", "https://myproj.com/abc123");
        Map<String, Object> record = new HashMap<>();
        record.put("kind", kind);
        record.put("data", data);
        ObjectMapper mapper = new ObjectMapper();
        JsonNode jsonNode = mapper.convertValue(record, JsonNode.class);
        TestActionRequest testActionRequest = TestActionRequest.builder().action(getCreateActionDto()).testPayload(jsonNode).build();

        ParsedAction response = this.sut.validateRegex(testActionRequest);
        assertEquals("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}", response.getUrl());
        assertEquals("Filter did not match the testPayload.", response.getErrors());
    }

    @Test
    public void should_throwError_given_missingRequiredPartsInKind() {
        Map<String, Object> data = new HashMap<>();
        data.put("uri", "https://myproj.com/abc123");
        Map<String, Object> record = new HashMap<>();
        record.put("kind", "common:petrel");
        record.put("data", data);
        ObjectMapper mapper = new ObjectMapper();
        JsonNode jsonNode = mapper.convertValue(record, JsonNode.class);
        TestActionRequest testActionRequest = TestActionRequest.builder().action(getCreateActionDto()).testPayload(jsonNode).build();

        ParsedAction response = this.sut.validateRegex(testActionRequest);
        assertEquals("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}", response.getUrl());
        assertEquals("Filter did not match the testPayload.", response.getErrors());
    }

    @Test
    public void should_throwError_given_kindFilterMismatch() {
        Map<String, Object> data = new HashMap<>();
        data.put("uri", "https://myproj.com/abc123");
        Map<String, Object> record = new HashMap<>();
        record.put("kind", "common:petrel:well:1.0.0");
        record.put("data", data);
        ObjectMapper mapper = new ObjectMapper();
        JsonNode jsonNode = mapper.convertValue(record, JsonNode.class);
        TestActionRequest testActionRequest = TestActionRequest.builder().action(getCreateActionDto()).testPayload(jsonNode).build();

        ParsedAction response = this.sut.validateRegex(testActionRequest);
        assertEquals("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}", response.getUrl());
        assertEquals("Filter did not match the testPayload.", response.getErrors());
    }

    private CreateActionDto getCreateActionDto() {
        Filter filter = Filter.builder()
                .entityType(Arrays.asList("regularheightfield", "project"))
                .source(Arrays.asList("petrel"))
                .version(Arrays.asList("*")).build();
        return CreateActionDto.builder()
                .name("test")
                .url("https://myapp.osdu.opengroup.org/action/{data.uri:^(?:[^\\/]*(?:\\/(?:\\/[^\\/]*\\/?)?)?([^?]+)(?:\\??.+)?)$}")
                .filter(filter).build();
    }
}
