/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.ddms.services;

import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.opengroup.osdu.register.utils.OpenApiFileUtils;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.apache.http.HttpStatus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.io.IOException;
import java.net.URI;

import static org.junit.Assert.*;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ConsumptionServiceTest {

    private Ddms ddms;

    @Mock
    private IDdmsRepository dmsRepository;
    @InjectMocks
    private ConsumptionService sut;

    private final String type = "testType";

    @Before
    public void setup() throws IOException {
        this.ddms = OpenApiFileUtils.getDmsWithOpenSpec("validSubsurfaceDdms.json", type);
    }

    @Test
    public void should_return_validUri_when_ddmsIdTypeFound() throws Exception {
        String id = "testDdmsId";
        String recordId = "testLocalId";

        when(this.dmsRepository.get(id)).thenReturn(this.ddms);
        URI redirectUri = this.sut.getSingleEntityRetrivalUri(id, type, recordId);
        assertNotNull(redirectUri);
        assertEquals("https://subsurface.data.osdu.opengroup.org/v1/wellbore/testLocalId", redirectUri.toString());
    }

    @Test
    public void should_throwException_when_ddmsId_notFound() {
        String id = "notCreatedId";
        when(this.dmsRepository.get(id)).thenReturn(null);

        try {
            this.sut.getSingleEntityRetrivalUri(id, "testType", "testLocalId");
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(HttpStatus.SC_NOT_FOUND, e.getError().getCode());
            assertEquals("Either the DDMS with the given id or the retrieval endpoint on DDMS interface was not found. ID: notCreatedId Type: testType", e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        } catch (Exception e) {
            fail("Should not throw this exception " + e.getMessage());
        }
    }

    @Test
    public void should_throwException_when_ddmsType_notFound() {
        String id = "testDdmsId";
        String type = "nonExistentType";
        when(this.dmsRepository.get(id)).thenReturn(this.ddms);

        try {
            this.sut.getSingleEntityRetrivalUri(id, type, anyString());
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(HttpStatus.SC_NOT_FOUND, e.getError().getCode());
            assertEquals("Either the DDMS with the given id or the retrieval endpoint on DDMS interface was not found. ID: testDdmsId Type: nonExistentType", e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        } catch (Exception e) {
            fail("Should not throw this exception " + e.getMessage());
        }
    }

    @Test
    public void should_throwException_when_ddmsUri_misconfigured() throws Exception {
        String id = "testDdmsId";
        String recordId = "testRecordId";
        this.ddms = OpenApiFileUtils.getDmsWithOpenSpec("validations/path/noGetMethod.json", "testType");
        when(this.dmsRepository.get(id)).thenReturn(this.ddms);

        try {
            this.sut.getSingleEntityRetrivalUri(id, type, recordId);
            fail("Should not succeed!");
        } catch (AppException e) {
            assertEquals(HttpStatus.SC_NOT_FOUND, e.getError().getCode());
            assertEquals("Either the DDMS with the given id or the retrieval endpoint on DDMS interface was not found. ID: testDdmsId Type: testType", e.getError().getMessage());
            assertEquals("Not found", e.getError().getReason());
        } catch (Exception e) {
            fail("Should not throw this exception " + e.getMessage());
        }
    }
}