/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.http.json.HttpResponseBodyMapper;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.http.IHttpClient;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.register.subscriber.model.ChallengeResponse;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.utils.AppServiceConfig;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class ChallengeResponseCheckTest {

    HmacSecret secret = new HmacSecret();

    @Mock
    private IHttpClient httpClient;

    @Mock
    private JaxRsDpsLog log;

    @Mock
    private DpsHeaders dpsHeaders;

    @Mock
    private ISignatureService signatureService;

    @Mock
    private AppServiceConfig serviceConfig;

    @Mock
    private HttpResponseBodyMapper responseBodyMapper;

    @InjectMocks
    private ChallengeResponseCheckSut sut;

    @Before
    public void setup() {
        when(serviceConfig.isLocalEnvironment()).thenReturn(false);
        secret.setValue("secret");
    }

    @Test
    public void should_returnFalse_when_returnedResponseIsNon200() {
        HttpResponse r = new HttpResponse();
        r.setResponseCode(400);
        when(httpClient.send(any())).thenReturn(r);
        Subscription s = new Subscription();
        s.setSecret(secret);
        s.setPushEndpoint("https://");
        boolean result = sut.runCreate(s);
        verify(httpClient).send(any());
        assertFalse(result);
    }

    @Test
    public void should_returnFalse_when_returnedResponseHashDoesNotMatchExpectedHash() {
        HttpResponse r = new HttpResponse();
        r.setResponseCode(200);
        r.setBody("{\"responseHash\":\"abc123\"}");
        Subscription s = new Subscription();
        HmacSecret secret = new HmacSecret();
        s.setSecret(secret);
        s.setPushEndpoint("https://");
        when(httpClient.send(any())).thenReturn(r);
        boolean result = sut.runCreate(s);
        verify(httpClient).send(any());
        assertFalse(result);
    }

    @Test
    public void should_returnFalse_when_returnedResponseThrowsAnError() {
        HttpResponse r = new HttpResponse();
        r.setResponseCode(200);
        r.setBody("{\"responseHash\":\"YmVlNmIyMGRkMzU1YzcxOTUwMTFjODU3OWZkMjU4ZWIzZDVkNjAzMGYyMjNjYWNkMzI2NWQxNWZjYjQ1MjU5Zg==\"}");
        when(httpClient.send(any())).thenThrow(new AppException(400, "", ""));
        Subscription s = new Subscription();
        s.setSecret(secret);
        s.setPushEndpoint("https://");
        boolean result = sut.runCreate(s);
        verify(httpClient).send(any());
        assertFalse(result);
    }

    @Test
    public void should_returnFalse_when_pushEndpointIsNotHttps() {
        HttpResponse r = new HttpResponse();
        r.setResponseCode(200);
        r.setBody("{\"responseHash\":\"YmVlNmIyMGRkMzU1YzcxOTUwMTFjODU3OWZkMjU4ZWIzZDVkNjAzMGYyMjNjYWNkMzI2NWQxNWZjYjQ1MjU5Zg==\"}");
        when(httpClient.send(any())).thenReturn(r);
        Subscription s = new Subscription();
        s.setSecret(secret);
        s.setPushEndpoint("http://");
        boolean result = sut.runCreate(s);
        verify(serviceConfig).isLocalEnvironment();
        assertFalse(result);
    }

    @Test
    public void should_returnTrue_when_returnedHashMatchesExpected() throws Exception {
        when(signatureService.getSignedSignature(any(), any())).thenReturn("signature");
        HttpResponse r = new HttpResponse();
        r.setResponseCode(200);
        r.setBody("{\"responseHash\":\"YmVlNmIyMGRkMzU1YzcxOTUwMTFjODU3OWZkMjU4ZWIzZDVkNjAzMGYyMjNjYWNkMzI2NWQxNWZjYjQ1MjU5Zg==\"}");
        when(httpClient.send(any())).thenReturn(r);
        Subscription s = new Subscription();
        HmacSecret secret = new HmacSecret();
        secret.setValue("secret");
        s.setSecret(secret);
        s.setPushEndpoint("https://");
        ChallengeResponse challengeResponse = new ChallengeResponse();
        challengeResponse.setResponseHash("YmVlNmIyMGRkMzU1YzcxOTUwMTFjODU3OWZkMjU4ZWIzZDVkNjAzMGYyMjNjYWNkMzI2NWQxNWZjYjQ1MjU5Zg==");
        when(responseBodyMapper.parseBody(r, ChallengeResponse.class)).thenReturn(challengeResponse);
        boolean result = sut.runCreate(s);
        assertTrue(result);
    }
}

class ChallengeResponseCheckSut extends ChallengeResponseCheck {

    @Override
    protected String getCrc() {
        return "crc";
    }

}