/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLogger;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;

import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class CreateSubscriptionTest {

    @Mock
    private ISubscriptionRepository subscriptionRepository;

    @Mock
    private ChallengeResponseCheck challengeResponseCheck;

    @Mock
    private AvailableTopics topics;

    @Mock
    private DpsHeaders dpsHeaders;

    @Mock
    private AuditLogger auditLogger;

    @InjectMocks
    private CreateSubscription sut;

    @Test
    public void should_throw400_when_givenSubscriptionWithTopicThatDoesNotExist() throws Exception {
        Subscription s = new Subscription();
        s.setTopic("a");
        when(topics.exists(s.getTopic())).thenReturn(false);

        try {
            sut.run(s);
            fail("Expected error");
        } catch (AppException e) {
            assertEquals(400, e.getError().getCode());
            assertTrue(e.getError().getMessage().startsWith("Given topic name does not match any"));
            assertEquals("Bad Request", e.getError().getReason());
        }
    }


    @Test
    public void should_throw400_when_givenSubscriptionWithEnpointThatFailsChallenge() throws Exception {
        Subscription s = new Subscription();
        when(topics.exists(any())).thenReturn(true);
        when(challengeResponseCheck.runCreate(s)).thenReturn(false);

        try {
            sut.run(s);
            fail("Expected error");
        } catch (AppException e) {
            assertEquals(400, e.getError().getCode());
            assertTrue(e.getError().getMessage().startsWith("Failed challenge response check to"));
            assertEquals("Bad Request", e.getError().getReason());
        }
    }

    @Test
    public void should_returnSubscriptionWithCreatedUserAndBase64EncodedId_when_givenValidInput() throws Exception {
        Subscription s = new Subscription();
        s.setTopic("abc");
        s.setPushEndpoint("123.com");
        when(topics.exists(any())).thenReturn(true);
        when(subscriptionRepository.create(s)).thenReturn(s);
        when(challengeResponseCheck.runCreate(s)).thenReturn(true);
        when(dpsHeaders.getUserEmail()).thenReturn("a@b.com");

        Subscription output = sut.run(s);
        verify(this.auditLogger).createSubscriptionSuccess(any());
        verify(this.subscriptionRepository).create(s);
        assertEquals("a@b.com", output.getCreatedBy());
        assertTrue(output.getNotificationId(), output.getNotificationId().startsWith("de-"));
        assertEquals("YWJjMTIzLmNvbQ==", output.getId());
    }
}
